@@ -19,7 +19,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.10.32"
+__version__ = "0.12.4"
 
 # Used with pypi checks and other messages related to pip
 _wandb_module = "wandb"
@@ -31,13 +31,7 @@ from wandb.errors import Error
 # This needs to be early as other modules call it.
 from wandb.errors.term import termsetup, termlog, termerror, termwarn
 
-PY3 = sys.version_info.major == 3 and sys.version_info.minor >= 6
-TYPE_CHECKING = False  # type: bool
-if PY3:
-    TYPE_CHECKING = True
-    from wandb import sdk as wandb_sdk
-else:
-    from wandb import sdk_py27 as wandb_sdk
+from wandb import sdk as wandb_sdk
 
 import wandb
 
@@ -52,6 +46,9 @@ finish = wandb_sdk.finish
 join = finish
 login = wandb_sdk.login
 helper = wandb_sdk.helper
+sweep = wandb_sdk.sweep
+controller = wandb_sdk.controller
+require = wandb_sdk.require
 Artifact = wandb_sdk.Artifact
 AlertLevel = wandb_sdk.AlertLevel
 Settings = wandb_sdk.Settings
@@ -63,7 +60,6 @@ from wandb.errors import CommError, UsageError
 _preinit = wandb_lib.preinit
 _lazyloader = wandb_lib.lazyloader
 from wandb import wandb_torch
-from wandb import util
 
 # Move this (keras.__init__ expects it at top level)
 from wandb.data_types import Graph
@@ -82,15 +78,13 @@ from wandb.data_types import Classes
 from wandb.data_types import JoinedTable
 
 from wandb.wandb_agent import agent
-from wandb.wandb_controller import sweep, controller
-
-from wandb import superagent
 
 # from wandb.core import *
 from wandb.viz import visualize
 from wandb import plot
 from wandb import plots  # deprecating this
 from wandb.integration.sagemaker import sagemaker_auth
+from wandb.sdk.internal import profiler
 
 
 # Used to make sure we don't use some code in the incorrect process context
@@ -193,12 +187,22 @@ def set_trace():
     pdb.set_trace()  # TODO: pass the parent stack...
 
 
+def load_ipython_extension(ipython):
+    ipython.register_magics(wandb.jupyter.WandBMagics)
+
+
+if wandb_sdk.lib.ipython.in_jupyter():
+    from IPython import get_ipython
+
+    load_ipython_extension(get_ipython())
+
 __all__ = [
     "__version__",
     "init",
     "setup",
     "save",
     "sweep",
+    "controller",
     "agent",
     "config",
     "log",
@@ -215,4 +219,5 @@ __all__ = [
     "Object3D",
     "Molecule",
     "Histogram",
+    "_enable",
 ]
