@@ -1,12 +1,4 @@
-import sys
-
-
-# TODO: consolidate dynamic imports
-PY3 = sys.version_info.major == 3 and sys.version_info.minor >= 6
-if PY3:
-    from wandb.sdk.internal.internal_api import Api as InternalApi
-else:
-    from wandb.sdk_py27.internal.internal_api import Api as InternalApi
+from wandb.sdk.internal.internal_api import Api as InternalApi
 
 
 class Api(object):
@@ -39,6 +31,10 @@ class Api(object):
     def app_url(self):
         return self.api.app_url
 
+    @property
+    def default_entity(self):
+        return self.api.default_entity
+
     @property
     def git(self):
         return self.api.git
@@ -49,6 +45,9 @@ class Api(object):
     def download_file(self, *args, **kwargs):
         return self.api.download_file(*args, **kwargs)
 
+    def download_write_file(self, *args, **kwargs):
+        return self.api.download_write_file(*args, **kwargs)
+
     def set_current_run_id(self, run_id):
         return self.api.set_current_run_id(run_id)
 
@@ -135,5 +134,20 @@ class Api(object):
     def upload_file_retry(self, *args, **kwargs):
         return self.api.upload_file_retry(*args, **kwargs)
 
+    def get_run_info(self, *args, **kwargs):
+        return self.api.get_run_info(*args, **kwargs)
+
+    def get_project_run_queues(self, *args, **kwargs):
+        return self.api.get_project_run_queues(*args, **kwargs)
+
+    def push_to_run_queue(self, *args, **kwargs):
+        return self.api.push_to_run_queue(*args, **kwargs)
+
+    def pop_from_run_queue(self, *args, **kwargs):
+        return self.api.pop_from_run_queue(*args, **kwargs)
+
+    def ack_run_queue_item(self, *args, **kwargs):
+        return self.api.ack_run_queue_item(*args, **kwargs)
+
 
 __all__ = ["Api"]
