@@ -21,6 +21,7 @@ SWEEP_PARAM_PATH = "WANDB_SWEEP_PARAM_PATH"
 SHOW_RUN = "WANDB_SHOW_RUN"
 DEBUG = "WANDB_DEBUG"
 SILENT = "WANDB_SILENT"
+QUIET = "WANDB_QUIET"
 INITED = "WANDB_INITED"
 DIR = "WANDB_DIR"
 # Deprecate DESCRIPTION in a future release
@@ -33,6 +34,7 @@ USER_EMAIL = "WANDB_USER_EMAIL"
 PROJECT = "WANDB_PROJECT"
 ENTITY = "WANDB_ENTITY"
 BASE_URL = "WANDB_BASE_URL"
+APP_URL = "WANDB_APP_URL"
 PROGRAM = "WANDB_PROGRAM"
 ARGS = "WANDB_ARGS"
 MODE = "WANDB_MODE"
@@ -63,6 +65,7 @@ ANONYMOUS = "WANDB_ANONYMOUS"
 JUPYTER = "WANDB_JUPYTER"
 CONFIG_DIR = "WANDB_CONFIG_DIR"
 CACHE_DIR = "WANDB_CACHE_DIR"
+DISABLE_SSL = "WANDB_INSECURE_DISABLE_SSL"
 
 # For testing, to be removed in future version
 USE_V1_ARTIFACTS = "_WANDB_USE_V1_ARTIFACTS"
@@ -99,6 +102,7 @@ def immutable_keys():
         HOST,
         CACHE_DIR,
         USE_V1_ARTIFACTS,
+        DISABLE_SSL,
     ]
 
 
@@ -121,6 +125,10 @@ def error_reporting_enabled():
     return _env_as_bool(ERROR_REPORTING, default=True)
 
 
+def ssl_disabled():
+    return _env_as_bool(DISABLE_SSL, default=False)
+
+
 def get_error_reporting(default=True, env=None):
     if env is None:
         env = os.environ
@@ -165,10 +173,10 @@ def get_ignore(default=None, env=None):
     if env is None:
         env = os.environ
 
-    if env.get(IGNORE, default):
-        return env.get(IGNORE, default).split(",")
+    if env.get(IGNORE):
+        return env.get(IGNORE).split(",")
     else:
-        return []
+        return default
 
 
 def get_project(default=None, env=None):
@@ -206,6 +214,13 @@ def get_base_url(default=None, env=None):
     return env.get(BASE_URL, default)
 
 
+def get_app_url(default=None, env=None):
+    if env is None:
+        env = os.environ
+
+    return env.get(APP_URL, default)
+
+
 def get_show_run(default=None, env=None):
     if env is None:
         env = os.environ
