@@ -11,9 +11,11 @@ from .wandb_config import Config  # noqa: F401
 from .wandb_history import History  # noqa: F401
 from .wandb_init import init  # noqa: F401
 from .wandb_login import login  # noqa: F401
+from .wandb_require import require  # noqa: F401
 from .wandb_run import finish  # noqa: F401
 from .wandb_save import save  # noqa: F401
 from .wandb_settings import Settings  # noqa: F401
 from .wandb_setup import setup  # noqa: F401
 from .wandb_summary import Summary  # noqa: F401
+from .wandb_sweep import controller, sweep  # noqa: F401
 from .wandb_watch import unwatch, watch  # noqa: F401
