@@ -2,23 +2,23 @@
 #
 # Contains common utility functions that enable
 # logging datasets and predictions to wandb.
+from collections.abc import Sequence
 import sys
+from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
 
 import wandb
 
-if wandb.TYPE_CHECKING:
 
-    from typing import TYPE_CHECKING, Callable, Dict, Union, Optional, List, Any
-    from collections.abc import Sequence
-
-    if TYPE_CHECKING:
-        from wandb.data_types import _TableIndex
+if TYPE_CHECKING:
+    from wandb.data_types import _TableIndex
 
 CAN_INFER_IMAGE_AND_VIDEO = sys.version_info.major == 3 and sys.version_info.minor >= 5
 
 
 class ValidationDataLogger(object):
-    """ValidationDataLogger is intended to be used inside of library integrations
+    """ValidationDataLogger helps to develop integrations which log  model predictions.
+
+    ValidationDataLogger is intended to be used inside of library integrations
     in order to facilitate the process of optionally building a validation dataset
     and logging periodic predictions against such validation data using WandB best
     practices.
@@ -100,11 +100,6 @@ class ValidationDataLogger(object):
         if indexes is None:
             assert targets is not None
             local_validation_table = wandb.Table(columns=[], data=[])
-            if isinstance(inputs, dict):
-                for col_name in inputs:
-                    local_validation_table.add_column(col_name, inputs[col_name])
-            else:
-                local_validation_table.add_column(input_col_name, inputs)
 
             if isinstance(targets, dict):
                 for col_name in targets:
@@ -112,6 +107,12 @@ class ValidationDataLogger(object):
             else:
                 local_validation_table.add_column(target_col_name, targets)
 
+            if isinstance(inputs, dict):
+                for col_name in inputs:
+                    local_validation_table.add_column(col_name, inputs[col_name])
+            else:
+                local_validation_table.add_column(input_col_name, inputs)
+
             if validation_row_processor is None and infer_missing_processors:
                 example_input = _make_example(inputs)
                 example_target = _make_example(targets)
@@ -164,9 +165,11 @@ class ValidationDataLogger(object):
         prediction_col_name: str = "output",
         val_ndx_col_name: str = "val_row",
         table_name: str = "validation_predictions",
-        commit: bool = False,
+        commit: bool = True,
     ) -> wandb.data_types.Table:
-        """Logs a set of predictions. Intended usage:
+        """Logs a set of predictions.
+
+        Intended usage:
 
         vl.log_predictions(vl.make_predictions(self.model.predict))
 
@@ -179,16 +182,13 @@ class ValidationDataLogger(object):
             table_name (str, optional): name of the prediction table. Defaults to "validation_predictions".
             commit (bool, optional): determines if commit should be called on the logged data. Defaults to False.
         """
-        if self.local_validation_artifact is not None:
-            self.local_validation_artifact.wait()
-
         pred_table = wandb.Table(columns=[], data=[])
-        pred_table.add_column(val_ndx_col_name, self.validation_indexes)
         if isinstance(predictions, dict):
             for col_name in predictions:
                 pred_table.add_column(col_name, predictions[col_name])
         else:
             pred_table.add_column(prediction_col_name, predictions)
+        pred_table.add_column(val_ndx_col_name, self.validation_indexes)
 
         if self.prediction_row_processor is None and self.infer_missing_processors:
             example_prediction = _make_example(predictions)
@@ -205,12 +205,12 @@ class ValidationDataLogger(object):
         if self.prediction_row_processor is not None:
             pred_table.add_computed_columns(self.prediction_row_processor)
 
-        wandb.log({table_name: pred_table})
+        wandb.log({table_name: pred_table}, commit=commit)
         return pred_table
 
 
 def _make_example(data: Any) -> Optional[Union[Dict, Sequence, Any]]:
-    """Used to make an example input, target, or output"""
+    """Used to make an example input, target, or output."""
     example: Optional[Union[Dict, Sequence, Any]]
 
     if isinstance(data, dict):
@@ -226,9 +226,9 @@ def _make_example(data: Any) -> Optional[Union[Dict, Sequence, Any]]:
 
 
 def _get_example_shape(example: Union[Sequence, Any]):
-    """Gets the shape of an object if applicable"""
+    """Gets the shape of an object if applicable."""
     shape = []
-    if hasattr(example, "__len__"):
+    if type(example) is not str and hasattr(example, "__len__"):
         length = len(example)
         shape = [length]
         if length > 0:
@@ -237,7 +237,7 @@ def _get_example_shape(example: Union[Sequence, Any]):
 
 
 def _bind(lambda_fn: Callable, **closure_kwargs: Any) -> Callable:
-    """Creates a closure around a lambda function by binding `closure_kwargs` to the function"""
+    """Creates a closure around a lambda function by binding `closure_kwargs` to the function."""
 
     def closure(*args: Any, **kwargs: Any) -> Any:
         _k = {}
@@ -253,8 +253,11 @@ def _infer_single_example_keyed_processor(
     class_labels_table: Optional["wandb.Table"] = None,
     possible_base_example: Optional[Union[Sequence, Any]] = None,
 ) -> Dict[str, Callable]:
-    """Infers a processor from a single example, with optional class_labels_table
-    and base_example. Base example is useful for cases such as segmentation masks"""
+    """Infers a processor from a single example.
+
+    Infers a processor from a single example with optional class_labels_table
+    and base_example. Base example is useful for cases such as segmentation masks
+    """
     shape = _get_example_shape(example)
     processors: Dict[str, Callable] = {}
     if (
@@ -267,6 +270,7 @@ def _infer_single_example_keyed_processor(
         )
         # Assume these are logits
         class_names = class_labels_table.get_column("label")
+
         processors["max_class"] = lambda n, d, p: class_labels_table.index_ref(  # type: ignore
             np.argmax(d)
         )
@@ -274,9 +278,13 @@ def _infer_single_example_keyed_processor(
         # processors["min_class"] = lambda n, d, p: class_labels_table.index_ref(  # type: ignore
         #     np.argmin(d)
         # )
-        processors["score"] = lambda n, d, p: {
-            class_names[i]: d[i] for i in range(shape[0])
-        }
+
+        values = np.unique(example)
+        is_one_hot = len(values) == 2 and set(values) == set([0, 1])
+        if not is_one_hot:
+            processors["score"] = lambda n, d, p: {
+                class_names[i]: d[i] for i in range(shape[0])
+            }
     elif (
         len(shape) == 1
         and shape[0] == 1
@@ -290,7 +298,7 @@ def _infer_single_example_keyed_processor(
             processors["class"] = lambda n, d, p: class_labels_table.index_ref(d[0]) if d[0] < len(class_labels_table.data) else d[0]  # type: ignore
         else:
             processors["val"] = lambda n, d, p: d[0]
-    elif len(shape) == 1 and shape[0] <= 10:
+    elif len(shape) == 1:
         np = wandb.util.get_module(
             "numpy", required="Infering processors require numpy",
         )
@@ -304,7 +312,11 @@ def _infer_single_example_keyed_processor(
             ]
         # just report the argmax and argmin
         processors["argmax"] = lambda n, d, p: np.argmax(d)
-        processors["argmin"] = lambda n, d, p: np.argmin(d)
+
+        values = np.unique(example)
+        is_one_hot = len(values) == 2 and set(values) == set([0, 1])
+        if not is_one_hot:
+            processors["argmin"] = lambda n, d, p: np.argmin(d)
     elif len(shape) == 2 and CAN_INFER_IMAGE_AND_VIDEO:
         if (
             class_labels_table is not None
@@ -341,7 +353,7 @@ def _infer_validation_row_processor(
     input_col_name: str = "input",
     target_col_name: str = "target",
 ) -> Callable:
-    """Infers the composit processor for the validation data"""
+    """Infers the composit processor for the validation data."""
     single_processors = {}
     if isinstance(example_input, dict):
         for key in example_input:
@@ -412,7 +424,7 @@ def _infer_prediction_row_processor(
     input_col_name: str = "input",
     output_col_name: str = "output",
 ) -> Callable:
-    """Infers the composit processor for the prediction output data"""
+    """Infers the composit processor for the prediction output data."""
     single_processors = {}
 
     if isinstance(example_prediction, dict):
