@@ -5,6 +5,7 @@ from gql.transport.requests import RequestsHTTPTransport  # type: ignore
 import datetime
 import ast
 import os
+from pkg_resources import parse_version  # type: ignore
 import json
 import yaml
 import re
@@ -18,6 +19,7 @@ if os.name == "posix" and sys.version_info[0] < 3:
 else:
     import subprocess  # type: ignore[no-redef]
 
+from copy import deepcopy
 import six
 from six import BytesIO
 import wandb
@@ -29,7 +31,7 @@ from wandb.apis.normalize import normalize_exceptions
 from wandb.errors import CommError, UsageError
 from wandb.integration.sagemaker import parse_sm_secrets
 from ..lib import retry
-from ..lib.filenames import DIFF_FNAME
+from ..lib.filenames import DIFF_FNAME, METADATA_FNAME
 from ..lib.git import GitRepo
 
 from .progress import Progress
@@ -112,6 +114,9 @@ class Api(object):
         self.upload_file_retry = normalize_exceptions(
             retry.retriable(retry_timedelta=retry_timedelta)(self.upload_file)
         )
+        self._client_id_mapping = {}
+
+        self.query_types, self.server_info_types = None, None
 
     def reauth(self):
         """Ensures the current api key is set in the transport"""
@@ -184,6 +189,10 @@ class Api(object):
     def app_url(self):
         return wandb.util.app_url(self.api_url)
 
+    @property
+    def default_entity(self):
+        return self.viewer().get("entity")
+
     def settings(self, key=None, section=None):
         """The settings overridden from the wandb/settings file.
 
@@ -271,6 +280,40 @@ class Api(object):
             assert run, "run must be specified"
         return (project, run)
 
+    @normalize_exceptions
+    def server_info_introspection(self):
+
+        query_string = """
+           query ProbeServerCapabilities {
+               QueryType: __type(name: "Query") {
+                   ...fieldData
+                }
+               ServerInfoType: __type(name: "ServerInfo") {
+                   ...fieldData
+                }
+            }
+
+            fragment fieldData on __Type {
+                fields {
+                    name
+                }
+            }
+        """
+
+        if self.query_types is None or self.server_info_types is None:
+            query = gql(query_string)
+            res = self.gql(query)
+
+            self.query_types = [
+                field.get("name", "")
+                for field in res.get("QueryType", {}).get("fields", [{}])
+            ]
+            self.server_info_types = [
+                field.get("name", "")
+                for field in res.get("ServerInfoType", {}).get("fields", [{}])
+            ]
+        return (self.query_types, self.server_info_types)
+
     @normalize_exceptions
     def viewer(self):
         query = gql(
@@ -296,12 +339,19 @@ class Api(object):
 
     @normalize_exceptions
     def viewer_server_info(self):
+        local_query = """
+                latestLocalVersionInfo {
+                    outOfDate
+                    latestVersionString
+                }
+        """
         cli_query = """
             serverInfo {
                 cliVersionInfo
+                _LOCAL_QUERY_
             }
         """
-        query_str = """
+        query_template = """
         query Viewer{
             viewer {
                 id
@@ -319,22 +369,25 @@ class Api(object):
             _CLI_QUERY_
         }
         """
-        query_new = gql(query_str.replace("_CLI_QUERY_", cli_query))
-        query_old = gql(query_str.replace("_CLI_QUERY_", ""))
+        query_types, server_info_types = self.server_info_introspection()
 
-        for query in query_new, query_old:
-            try:
-                res = self.gql(query)
-            except UsageError as e:
-                raise (e)
-            except Exception as e:
-                # graphql schema exception is generic
-                err = e
-                continue
-            err = None
-            break
-        if err:
-            raise (err)
+        cli_version_exists = (
+            "serverInfo" in query_types and "cliVersionInfo" in server_info_types
+        )
+
+        local_version_exists = (
+            "serverInfo" in query_types
+            and "latestLocalVersionInfo" in server_info_types
+        )
+
+        cli_query_string = "" if not cli_version_exists else cli_query
+        local_query_string = "" if not local_version_exists else local_query
+
+        query_string = query_template.replace("_CLI_QUERY_", cli_query_string).replace(
+            "_LOCAL_QUERY_", local_query_string
+        )
+        query = gql(query_string)
+        res = self.gql(query)
         return res.get("viewer") or {}, res.get("serverInfo") or {}
 
     @normalize_exceptions
@@ -370,7 +423,7 @@ class Api(object):
 
     @normalize_exceptions
     def project(self, project, entity=None):
-        """Retrive project
+        """Retrieve project
 
         Arguments:
             project (str): The project to get details for
@@ -571,12 +624,22 @@ class Api(object):
         """
         query = gql(
             """
-        query Model($name: String!, $entity: String!, $run: String!) {
+        query Model(
+            $name: String!,
+            $entity: String!,
+            $run: String!,
+            $pattern: String!,
+            $includeConfig: Boolean!,
+        ) {
             model(name: $name, entityName: $entity) {
                 bucket(name: $run) {
-                    config
-                    commit
-                    files(names: ["wandb-metadata.json", "diff.patch"]) {
+                    config @include(if: $includeConfig)
+                    commit @include(if: $includeConfig)
+                    files(pattern: $pattern) {
+                        pageInfo {
+                            hasNextPage
+                            endCursor
+                        }
                         edges {
                             node {
                                 name
@@ -590,26 +653,49 @@ class Api(object):
         """
         )
 
-        response = self.gql(
-            query, variable_values={"name": project, "run": run, "entity": entity}
-        )
-        if response["model"] == None:
-            raise CommError("Run {}/{}/{} not found".format(entity, project, run))
-        run = response["model"]["bucket"]
-        commit = run["commit"]
-        config = json.loads(run["config"] or "{}")
+        variable_values = {
+            "name": project,
+            "run": run,
+            "entity": entity,
+            "includeConfig": True,
+        }
+
+        commit = ""
+        config = {}
         patch = None
         metadata = {}
-        if len(run["files"]["edges"]) > 0:
-            for file_edge in run["files"]["edges"]:
-                name = file_edge["node"]["name"]
-                url = file_edge["node"]["directUrl"]
-                res = requests.get(url)
-                res.raise_for_status()
-                if name == "wandb-metadata.json":
-                    metadata = res.json()
-                elif name == "diff.patch":
-                    patch = res.text
+
+        # If we use the `names` paramter on the `files` node, then the server
+        # will helpfully give us and 'open' file handle to the files that don't
+        # exist. This is so that we can upload data to it. However, in this
+        # case, we just want to download that file and not upload to it, so
+        # let's instead query for the files that do exist using `pattern`
+        # (with no wildcards).
+        #
+        # Unfortunately we're unable to construct a single pattern that matches
+        # our 2 files, we would need something like regex for that.
+        for filename in [DIFF_FNAME, METADATA_FNAME]:
+            variable_values["pattern"] = filename
+            response = self.gql(query, variable_values=variable_values)
+            if response["model"] == None:
+                raise CommError("Run {}/{}/{} not found".format(entity, project, run))
+            run = response["model"]["bucket"]
+            # we only need to fetch this config once
+            if variable_values["includeConfig"]:
+                commit = run["commit"]
+                config = json.loads(run["config"] or "{}")
+                variable_values["includeConfig"] = False
+            if run["files"] is not None:
+                for file_edge in run["files"]["edges"]:
+                    name = file_edge["node"]["name"]
+                    url = file_edge["node"]["directUrl"]
+                    res = requests.get(url)
+                    res.raise_for_status()
+                    if name == METADATA_FNAME:
+                        metadata = res.json()
+                    elif name == DIFF_FNAME:
+                        patch = res.text
+
         return (commit, config, patch, metadata)
 
     @normalize_exceptions
@@ -734,11 +820,144 @@ class Api(object):
         return response["upsertModel"]["model"]
 
     @normalize_exceptions
-    def pop_from_run_queue(self, entity=None, project=None):
+    def get_project_run_queues(self, entity, project):
+        query = gql(
+            """
+        query Project($entity: String!, $projectName: String!){
+            project(entityName: $entity, name: $projectName) {
+                runQueues {
+                    id
+                    name
+                    createdBy
+                    access
+                }
+            }
+        }
+        """
+        )
+        variable_values = {
+            "projectName": project,
+            "entity": entity,
+        }
+
+        res = self.gql(query, variable_values)
+        if res.get("project") is None:
+            raise Exception(
+                "Error fetching run queues for {}/{} check that you have access to this entity and project".format(
+                    entity, project
+                )
+            )
+
+        return res["project"]["runQueues"]
+
+    @normalize_exceptions
+    def create_run_queue(self, entity, project, queue_name, access):
+        query = gql(
+            """
+        mutation createRunQueue($entity: String!, $project: String!, $queueName: String!, $access: RunQueueAccessType!){
+            createRunQueue(
+                input: {
+                    entityName: $entity,
+                    projectName: $project,
+                    queueName: $queueName,
+                    access: $access
+                }
+            ) {
+                success
+                queueID
+            }
+            
+        }
+        """
+        )
+        variable_values = {
+            "project": project,
+            "entity": entity,
+            "access": access,
+            "queueName": queue_name,
+        }
+        return self.gql(query, variable_values)["createRunQueue"]
+
+    @normalize_exceptions
+    def push_to_run_queue(self, queue_name, launch_spec):
+        # TODO(kdg): add pushToRunQueueByName to avoid this extra query
+        entity = launch_spec["entity"]
+        project = launch_spec["project"]
+        queues_found = self.get_project_run_queues(entity, project)
+        matching_queues = [
+            q
+            for q in queues_found
+            if q["name"] == queue_name
+            # ensure user has access to queue
+            and (q["access"] == "PROJECT" or q["createdBy"] == self.default_entity)
+        ]
+        if not matching_queues:
+            # in the case of a missing default queue. create it
+            if queue_name == "default":
+                wandb.termlog(
+                    "No default queue existing for {}/{} creating one.".format(
+                        entity, project
+                    )
+                )
+                res = self.create_run_queue(
+                    launch_spec["entity"],
+                    launch_spec["project"],
+                    queue_name,
+                    access="PROJECT",
+                )
+
+                if res is None or res.get("queueID") is None:
+                    wandb.termerror(
+                        "Unable to create default queue for {}/{}. Run could not be added to a queue".format(
+                            entity, project
+                        )
+                    )
+                    return
+                queue_id = res["queueID"]
+
+            else:
+                wandb.termwarn(
+                    "Unable to push to run queue {}. Queue not found.".format(
+                        queue_name
+                    )
+                )
+                return None
+        elif len(matching_queues) > 1:
+            wandb.termerror(
+                "Unable to push to run queue {}. More than one queue found with this name.".format(
+                    queue_name
+                )
+            )
+            return None
+        else:
+            queue_id = matching_queues[0]["id"]
+
         mutation = gql(
             """
-        mutation popFromRunQueue($entity: String!, $project: String!)  {
-            popFromRunQueue(input: { entityName: $entity, projectName: $project }) {
+        mutation pushToRunQueue($queueID: ID!, $runSpec: JSONString!) {
+            pushToRunQueue(
+                input: {
+                    queueID: $queueID,
+                    runSpec: $runSpec
+                }
+            ) {
+                runQueueItemId
+            }
+        }
+        """
+        )
+        spec_json = json.dumps(launch_spec)
+        response = self.gql(
+            mutation, variable_values={"queueID": queue_id, "runSpec": spec_json}
+        )
+        return response["pushToRunQueue"]
+
+    @normalize_exceptions
+    def pop_from_run_queue(self, queue_name, entity=None, project=None):
+        mutation = gql(
+            """
+        mutation popFromRunQueue($entity: String!, $project: String!, $queueName: String!)  {
+            popFromRunQueue(input: { entityName: $entity, projectName: $project, queueName: $queueName }) {
                 runQueueItemId
                 runSpec
             }
@@ -746,10 +965,35 @@ class Api(object):
         """
         )
         response = self.gql(
-            mutation, variable_values={"entity": entity, "project": project}
+            mutation,
+            variable_values={
+                "entity": entity,
+                "project": project,
+                "queueName": queue_name,
+            },
         )
         return response["popFromRunQueue"]
 
+    @normalize_exceptions
+    def ack_run_queue_item(self, item_id, run_id=None):
+        mutation = gql(
+            """
+        mutation ackRunQueueItem($itemId: ID!, $runId: String!)  {
+            ackRunQueueItem(input: { runQueueItemId: $itemId, runName: $runId }) {
+                success
+            }
+        }
+        """
+        )
+        response = self.gql(
+            mutation, variable_values={"itemId": item_id, "runId": str(run_id)}
+        )
+        if not response["ackRunQueueItem"]["success"]:
+            raise CommError(
+                "Error acking run queue item. Item may have already been acknowledged by another process"
+            )
+        return response["ackRunQueueItem"]["success"]
+
     @normalize_exceptions
     def upsert_run(
         self,
@@ -898,6 +1142,50 @@ class Api(object):
 
         return response["upsertBucket"]["bucket"], response["upsertBucket"]["inserted"]
 
+    @normalize_exceptions
+    def get_run_info(self, entity, project, name):
+        query = gql(
+            """
+        query Run($project: String!, $entity: String!, $name: String!) {
+            project(name: $project, entityName: $entity) {
+                run(name: $name) {
+                    runInfo {
+                        program
+                        args
+                        os
+                        python
+                        colab
+                        executable
+                        codeSaved
+                        cpuCount
+                        gpuCount
+                        gpu
+                        git {
+                            remote
+                            commit
+                        }
+                    }
+                }
+            }
+        }
+        """
+        )
+        variable_values = {"project": project, "entity": entity, "name": name}
+        res = self.gql(query, variable_values)
+        if res.get("project") is None:
+            raise CommError(
+                "Error fetching run info for {}/{}/{}. Check that this project exists and you have access to this entity and project".format(
+                    entity, project, name
+                )
+            )
+        elif res["project"].get("run") is None:
+            raise CommError(
+                "Error fetching run info for {}/{}/{}. Check that this run id exists".format(
+                    entity, project, name
+                )
+            )
+        return res["project"]["run"]["runInfo"]
+
     @normalize_exceptions
     def upload_urls(self, project, files, run=None, entity=None, description=None):
         """Generate temporary resumeable upload urls
@@ -1126,7 +1414,8 @@ class Api(object):
             if status_code in (308, 408, 409, 429, 500, 502, 503, 504) or isinstance(
                 e, (requests.exceptions.Timeout, requests.exceptions.ConnectionError)
             ):
-                util.sentry_reraise(retry.TransientException(exc=e))
+                e = retry.TransientError(exc=e)
+                six.reraise(type(e), e, sys.exc_info()[2])
             else:
                 util.sentry_reraise(e)
 
@@ -1240,6 +1529,45 @@ class Api(object):
         else:
             return json.loads(response["agentHeartbeat"]["commands"])
 
+    @staticmethod
+    def _validate_config_and_fill_distribution(config):
+        # verify that parameters are well specified.
+        # TODO(dag): deprecate this in favor of jsonschema validation once
+        # apiVersion 2 is released and local controller is integrated with
+        # wandb/client.
+
+        # avoid modifying the original config dict in
+        # case it is reused outside the calling func
+        config = deepcopy(config)
+
+        # explicitly cast to dict in case config was passed as a sweepconfig
+        # sweepconfig does not serialize cleanly to yaml and breaks graphql
+        # but it is a subclass of dict, so this conversion is clean
+        config = dict(config)
+
+        if "parameters" not in config:
+            raise ValueError("sweep config must have a parameters section")
+
+        for parameter_name in config["parameters"]:
+            parameter = config["parameters"][parameter_name]
+            if "min" in parameter and "max" in parameter:
+                if "distribution" not in parameter:
+                    if isinstance(parameter["min"], int) and isinstance(
+                        parameter["max"], int
+                    ):
+                        parameter["distribution"] = "int_uniform"
+                    elif isinstance(parameter["min"], float) and isinstance(
+                        parameter["max"], float
+                    ):
+                        parameter["distribution"] = "uniform"
+                    else:
+                        raise ValueError(
+                            "Parameter %s is ambiguous, please specify bounds as both floats (for a float_"
+                            "uniform distribution) or ints (for an int_uniform distribution)."
+                            % parameter_name
+                        )
+        return config
+
     @normalize_exceptions
     def upsert_sweep(
         self,
@@ -1254,7 +1582,7 @@ class Api(object):
         """Upsert a sweep object.
 
         Arguments:
-            config (str): sweep config (will be converted to yaml)
+            config (dict): sweep config (will be converted to yaml)
         """
         project_query = """
                     project {
@@ -1327,6 +1655,8 @@ class Api(object):
         # TODO(dag): replace this with a query for protocol versioning
         mutations = [mutation_3, mutation_2, mutation_1]
 
+        config = self._validate_config_and_fill_distribution(config)
+
         for mutation in mutations:
             try:
                 response = self.gql(
@@ -1597,6 +1927,8 @@ class Api(object):
         artifact_type_name,
         artifact_collection_name,
         digest,
+        client_id=None,
+        sequence_client_id=None,
         entity_name=None,
         project_name=None,
         run_name=None,
@@ -1607,6 +1939,15 @@ class Api(object):
         distributed_id=None,
         is_user_created=False,
     ):
+        # TODO: Ignore clientID and sequenceClientID if server can't handle it
+        _, server_info = self.viewer_server_info()
+        max_cli_version = server_info.get("cliVersionInfo", {}).get(
+            "max_cli_version", None
+        )
+        can_handle_client_id = max_cli_version is None or parse_version(
+            "0.11.0"
+        ) <= parse_version(max_cli_version)
+
         mutation = gql(
             """
         mutation CreateArtifact(
@@ -1621,6 +1962,8 @@ class Api(object):
             $aliases: [ArtifactAliasInput!],
             $metadata: JSONString,
             %s
+            %s
+            %s
         ) {
             createArtifact(input: {
                 artifactTypeName: $artifactTypeName,
@@ -1635,6 +1978,8 @@ class Api(object):
                 aliases: $aliases,
                 metadata: $metadata,
                 %s
+                %s
+                %s
             }) {
                 artifact {
                     id
@@ -1659,8 +2004,12 @@ class Api(object):
             # For backwards compatibility with older backends that don't support
             # distributed writers.
             (
-                "$distributedID: String" if distributed_id else "",
-                "distributedID: $distributedID" if distributed_id else "",
+                "$distributedID: String," if distributed_id else "",
+                "$clientID: ID!," if can_handle_client_id else "",
+                "$sequenceClientID: ID!," if can_handle_client_id else "",
+                "distributedID: $distributedID," if distributed_id else "",
+                "clientID: $clientID," if can_handle_client_id else "",
+                "sequenceClientID: $sequenceClientID," if can_handle_client_id else "",
             )
         )
 
@@ -1679,6 +2028,8 @@ class Api(object):
                 "runName": run_name,
                 "artifactTypeName": artifact_type_name,
                 "artifactCollectionNames": [artifact_collection_name],
+                "clientID": client_id,
+                "sequenceClientID": sequence_client_id,
                 "digest": digest,
                 "description": description,
                 "aliases": [alias for alias in aliases],
@@ -1855,6 +2206,32 @@ class Api(object):
             response["updateArtifactManifest"]["artifactManifest"]["file"],
         )
 
+    def _resolve_client_id(
+        self, client_id,
+    ):
+
+        if client_id in self._client_id_mapping:
+            return self._client_id_mapping[client_id]
+
+        query = gql(
+            """
+            query ClientIDMapping($clientID: ID!) {
+                clientIDMapping(clientID: $clientID) {
+                    serverID
+                }
+            }
+        """
+        )
+        response = self.gql(query, variable_values={"clientID": client_id,},)
+        server_id = None
+        if response is not None:
+            client_id_mapping = response.get("clientIDMapping")
+            if client_id_mapping is not None:
+                server_id = client_id_mapping.get("serverID")
+                if server_id is not None:
+                    self._client_id_mapping[client_id] = server_id
+        return server_id
+
     @normalize_exceptions
     def create_artifact_files(self, artifact_files):
         mutation = gql(
