@@ -4,15 +4,12 @@
 static settings.
 """
 
-import wandb
+from typing import TYPE_CHECKING
 
-if wandb.TYPE_CHECKING:
-    from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from typing import Any, Dict, Iterable, Optional, Union
 
-    if TYPE_CHECKING:
-        from typing import Any, Dict, Iterable, Optional, Union
-
-        SettingsDict = Dict[str, Union[str, float]]
+    SettingsDict = Dict[str, Union[str, float]]
 
 
 class SettingsStatic(object):
