@@ -9,6 +9,7 @@ import socket
 import sys
 import threading
 import time
+from typing import TYPE_CHECKING
 
 import six
 from six.moves import queue
@@ -18,21 +19,17 @@ from wandb.viz import custom_chart_panel_config, CustomChart
 
 from . import run as internal_run
 
-if wandb.TYPE_CHECKING:
-    from typing import TYPE_CHECKING
 
-    if TYPE_CHECKING:
-        from ..interface.interface import BackendSender
-        from .settings_static import SettingsStatic
-        from typing import Dict, List, Optional
-        from wandb.proto.wandb_internal_pb2 import RunRecord
-        from six.moves.queue import PriorityQueue
-        from tensorboard.compat.proto.event_pb2 import ProtoEvent
-        from tensorboard.backend.event_processing.event_file_loader import (
-            EventFileLoader,
-        )
+if TYPE_CHECKING:
+    from ..interface.interface import BackendSender
+    from .settings_static import SettingsStatic
+    from typing import Dict, List, Optional
+    from wandb.proto.wandb_internal_pb2 import RunRecord
+    from six.moves.queue import PriorityQueue
+    from tensorboard.compat.proto.event_pb2 import ProtoEvent
+    from tensorboard.backend.event_processing.event_file_loader import EventFileLoader
 
-        HistoryDict = Dict[str, object]
+    HistoryDict = Dict[str, object]
 
 # Give some time for tensorboard data to be flushed
 SHUTDOWN_DELAY = 5
@@ -71,7 +68,7 @@ def is_tfevents_file_created_by(path: str, hostname: str, start_time: float) ->
     if not path:
         raise ValueError("Path must be a nonempty string")
     basename = os.path.basename(path)
-    if basename.endswith(".profile-empty"):
+    if basename.endswith(".profile-empty") or basename.endswith(".sagemaker-uploaded"):
         return False
     fname_components = basename.split(".")
     try:
@@ -201,7 +198,7 @@ class TBDirWatcher(object):
         self._generator = self.directory_watcher.DirectoryWatcher(
             logdir, self._loader(save, namespace), self._is_our_tfevents_file
         )
-        self._thread = threading.Thread(target=self._thread_body)
+        self._thread = threading.Thread(target=self._thread_except_body)
         self._first_event_timestamp = None
         self._shutdown = threading.Event()
         self._queue = queue
@@ -273,6 +270,13 @@ class TBDirWatcher(object):
             if not self._shutdown.is_set() and not shutdown_call:
                 time.sleep(ERROR_DELAY)
 
+    def _thread_except_body(self) -> None:
+        try:
+            self._thread_body()
+        except Exception as e:
+            logger.exception("generic exception in TBDirWatcher thread")
+            raise e
+
     def _thread_body(self) -> None:
         """Check for new events every second"""
         shutdown_time: "Optional[float]" = None
@@ -337,7 +341,7 @@ class TBEventConsumer(object):
     ) -> None:
         self._tbwatcher = tbwatcher
         self._queue = queue
-        self._thread = threading.Thread(target=self._thread_body)
+        self._thread = threading.Thread(target=self._thread_except_body)
         self._shutdown = threading.Event()
         self.tb_history = TBHistory()
         self._delay = delay
@@ -369,6 +373,13 @@ class TBEventConsumer(object):
                 self._save_row(item,)
         self._thread.join()
 
+    def _thread_except_body(self) -> None:
+        try:
+            self._thread_body()
+        except Exception as e:
+            logger.exception("generic exception in TBEventConsumer thread")
+            raise e
+
     def _thread_body(self) -> None:
         while True:
             try:
@@ -442,14 +453,14 @@ class TBHistory(object):
         # A single tensorboard step may have too much data
         # we just drop the largest keys in the step if it does.
         # TODO: we could flush the data across multiple steps
-        if self._step_size > util.MAX_LINE_SIZE:
+        if self._step_size > util.MAX_LINE_BYTES:
             metrics = [(k, sys.getsizeof(v)) for k, v in self._data.items()]
             metrics.sort(key=lambda t: t[1], reverse=True)
             bad = 0
             dropped_keys = []
             for k, v in metrics:
                 # TODO: (cvp) Added a buffer of 100KiB, this feels rather brittle.
-                if self._step_size - bad < util.MAX_LINE_SIZE - 100000:
+                if self._step_size - bad < util.MAX_LINE_BYTES - 100000:
                     break
                 else:
                     bad += v
