@@ -1,9 +1,13 @@
 #
 # -*- coding: utf-8 -*-
-"""
-wandb.init() indicates the beginning of a new run. In an ML training pipeline,
-you could add wandb.init() to the beginning of your training script as well as
-your evaluation script, and each piece steps would be tracked as a run in W&B.
+"""Defines wandb.init() and associated classes and methods.
+
+`wandb.init()` indicates the beginning of a new run. In an ML training pipeline,
+you could add `wandb.init()` to the beginning of your training script as well as
+your evaluation script, and each step would be tracked as a run in W&B.
+
+For more on using `wandb.init()`, including code snippets, check out our
+[guide and FAQs](https://docs.wandb.ai/guides/track/launch).
 """
 
 from __future__ import print_function
@@ -13,8 +17,10 @@ import logging
 import os
 import platform
 import sys
+import tempfile
 import time
 import traceback
+from typing import Any, Dict, Optional, Sequence, Union
 
 import shortuuid  # type: ignore
 import six
@@ -30,11 +36,9 @@ from .backend.backend import Backend
 from .lib import filesystem, ipython, module, reporting, telemetry
 from .lib import RunDisabled, SummaryDisabled
 from .wandb_helper import parse_config
-from .wandb_run import Run
+from .wandb_run import Run, TeardownHook, TeardownStage
 from .wandb_settings import Settings
 
-if wandb.TYPE_CHECKING:  # type: ignore
-    from typing import Optional, Union, List, Sequence, Dict, Any  # noqa: F401
 
 logger = None  # logger configured during wandb.init()
 
@@ -70,12 +74,17 @@ class _WandbInit(object):
         self._wl = None
         self._reporter = None
         self._use_sagemaker = None
+
+        self._set_init_name = None
+        self._set_init_tags = None
+        self._set_init_id = None
+        self._set_init_config = None
         self.notebook = None
 
     def setup(self, kwargs) -> None:
-        """
-        Complete setup for wandb.init(). This includes parsing all arguments,
-        applying them with settings and enabling logging.
+        """Completes setup for `wandb.init()`.
+
+        This includes parsing all arguments, applying them with settings and enabling logging.
         """
         self.kwargs = kwargs
 
@@ -93,7 +102,9 @@ class _WandbInit(object):
             settings=settings.duplicate().freeze()
         )
 
-        sm_config: Dict = {} if settings.sagemaker_disable else sagemaker.parse_sm_config()
+        sm_config: Dict = (
+            {} if settings.sagemaker_disable else sagemaker.parse_sm_config()
+        )
         if sm_config:
             sm_api_key = sm_config.get("wandb_api_key", None)
             sm_run, sm_env = sagemaker.parse_sm_resources()
@@ -104,9 +115,10 @@ class _WandbInit(object):
                 wandb.setup(settings=settings)
             settings._apply_setup(sm_run)
             self._use_sagemaker = True
-
+        self._set_init_telemetry_attrs(kwargs)
         # Remove parameters that are not part of settings
         init_config = kwargs.pop("config", None) or dict()
+
         config_include_keys = kwargs.pop("config_include_keys", None)
         config_exclude_keys = kwargs.pop("config_exclude_keys", None)
 
@@ -152,7 +164,12 @@ class _WandbInit(object):
         settings._apply_init_login(kwargs)
 
         if not settings._offline and not settings._noop:
-            wandb_login._login(anonymous=anonymous, force=force, _disable_warning=True)
+            wandb_login._login(
+                anonymous=anonymous,
+                force=force,
+                _disable_warning=True,
+                _silent=(settings._quiet or settings._silent) is True,
+            )
 
         # apply updated global state after login was handled
         settings._apply_settings(wandb.setup()._settings)
@@ -187,12 +204,12 @@ class _WandbInit(object):
         # normally this happens on the run object
         logger.info("tearing down wandb.init")
         for hook in self._teardown_hooks:
-            hook()
+            hook.call()
 
     def _enable_logging(self, log_fname, run_id=None):
-        """
-        Enable logging to the global debug log.  This adds a run_id to the log,
-        in case of muliple processes on the same machine.
+        """Enables logging to the global debug log.
+
+        This adds a run_id to the log, in case of muliple processes on the same machine.
         Currently there is no way to disable logging after it's enabled.
         """
         handler = logging.FileHandler(log_fname)
@@ -222,7 +239,10 @@ class _WandbInit(object):
         # TODO: make me configurable
         logger.setLevel(logging.DEBUG)
         self._teardown_hooks.append(
-            lambda: (handler.close(), logger.removeHandler(handler))
+            TeardownHook(
+                lambda: (handler.close(), logger.removeHandler(handler)),
+                TeardownStage.LATE,
+            )
         )
 
     def _safe_symlink(self, base, target, name, delete=False):
@@ -260,12 +280,12 @@ class _WandbInit(object):
             self.backend.interface.publish_resume()
 
     def _jupyter_teardown(self):
-        """Teardown hooks and display saving, called with wandb.finish"""
+        """Teardown hooks and display saving, called with wandb.finish."""
         ipython = self.notebook.shell
         self.notebook.save_history()
         if self.notebook.save_ipynb():
-            self.run.log_code(root=None)
-            logger.info("saved code and history")
+            res = self.run.log_code(root=None)
+            logger.info("saved code and history: %s", res)
         logger.info("cleaning up jupyter logic")
         # because of how we bind our methods we manually find them to unregister
         for hook in ipython.events.callbacks["pre_run_cell"]:
@@ -278,10 +298,9 @@ class _WandbInit(object):
         del ipython.display_pub._orig_publish
 
     def _jupyter_setup(self, settings):
-        """Add magic, hooks, and session history saving"""
+        """Add hooks, and session history saving."""
         self.notebook = wandb.jupyter.Notebook(settings)
         ipython = self.notebook.shell
-        ipython.register_magics(wandb.jupyter.WandBMagics)
 
         # Monkey patch ipython publish to capture displayed outputs
         if not hasattr(ipython.display_pub, "_orig_publish"):
@@ -291,7 +310,9 @@ class _WandbInit(object):
 
             ipython.events.register("pre_run_cell", self._resume_backend)
             ipython.events.register("post_run_cell", self._pause_backend)
-            self._teardown_hooks.append(self._jupyter_teardown)
+            self._teardown_hooks.append(
+                TeardownHook(self._jupyter_teardown, TeardownStage.EARLY)
+            )
 
         def publish(data, metadata=None, **kwargs):
             ipython.display_pub._orig_publish(data, metadata=metadata, **kwargs)
@@ -302,8 +323,7 @@ class _WandbInit(object):
         ipython.display_pub.publish = publish
 
     def _log_setup(self, settings):
-        """Set up logging from settings."""
-
+        """Sets up logging from settings."""
         filesystem._safe_makedirs(os.path.dirname(settings.log_user))
         filesystem._safe_makedirs(os.path.dirname(settings.log_internal))
         filesystem._safe_makedirs(os.path.dirname(settings.sync_file))
@@ -350,7 +370,7 @@ class _WandbInit(object):
         drun.disabled = True
         drun.id = shortuuid.uuid()
         drun.name = "dummy-" + drun.id
-        drun.dir = "/"
+        drun.dir = tempfile.gettempdir()
         module.set_global(
             run=drun,
             config=drun.config,
@@ -392,11 +412,7 @@ class _WandbInit(object):
                         last_id
                     )
                 )
-                jupyter = (
-                    s._jupyter
-                    and not s._silent
-                    and ipython._get_python_type() == "jupyter"
-                )
+                jupyter = s._jupyter and not s._silent and ipython.in_jupyter()
                 if jupyter:
                     ipython.display_html(
                         "Finishing last run (ID:{}) before initializing another...".format(
@@ -408,7 +424,7 @@ class _WandbInit(object):
 
                 if jupyter:
                     ipython.display_html(
-                        "...Successfully finished last run (ID:{}). Initializing new run:<br/><br/>".format(
+                        "Successfully finished last run (ID:{}). Initializing new run:<br/>".format(
                             last_id
                         )
                     )
@@ -453,6 +469,17 @@ class _WandbInit(object):
             run._telemetry_imports(tel.imports_init)
             if self._use_sagemaker:
                 tel.feature.sagemaker = True
+            if self._set_init_config:
+                tel.feature.set_init_config = True
+            if self._set_init_name:
+                tel.feature.set_init_name = True
+            if self._set_init_id:
+                tel.feature.set_init_id = True
+            if self._set_init_tags:
+                tel.feature.set_init_tags = True
+
+            if self.settings.launch:
+                tel.feature.launch = True
 
             if active_start_method == "spawn":
                 tel.env.start_spawn = True
@@ -487,6 +514,10 @@ class _WandbInit(object):
             run_proto = backend.interface._make_run(run)
             backend.interface._publish_run(run_proto)
             run._set_run_obj_offline(run_proto)
+            if s.resume:
+                wandb.termwarn(
+                    f"`resume` will be ignored since W&B syncing is set to `offline`. Starting a new run with run id {run.id}."
+                )
         else:
             logger.info("communicating current version")
             ret = backend.interface.communicate_check_version(
@@ -503,7 +534,8 @@ class _WandbInit(object):
             run._on_init()
             logger.info("communicating run to backend with 30 second timeout")
             ret = backend.interface.communicate_run(run, timeout=30)
-            error_message = None
+
+            error_message: Optional[str] = None
             if not ret:
                 logger.error("backend process timed out")
                 error_message = "Error communicating with wandb process"
@@ -515,6 +547,7 @@ class _WandbInit(object):
                 error_message = ret.error.message
             if error_message:
                 logger.error("encountered error: {}".format(error_message))
+
                 # Shutdown the backend and get rid of the logger
                 # we don't need to do console cleanup at this point
                 backend.cleanup()
@@ -554,6 +587,17 @@ class _WandbInit(object):
         logger.info("run started, returning control to user process")
         return run
 
+    def _set_init_telemetry_attrs(self, kwargs):
+        # config not set here because the
+        if kwargs.get("name"):
+            self._set_init_name = True
+        if kwargs.get("id"):
+            self._set_init_id = True
+        if kwargs.get("tags"):
+            self._set_init_tags = True
+        if kwargs.get("config"):
+            self._set_init_config = True
+
 
 def getcaller():
     # py2 doesnt have stack_info
@@ -588,8 +632,7 @@ def init(
     id=None,
     settings: Union[Settings, Dict[str, Any], None] = None,
 ) -> Union[Run, RunDisabled, None]:
-    """
-    Start a new tracked run with `wandb.init()`.
+    """Starts a new run to track and log to W&B.
 
     In an ML training pipeline, you could add `wandb.init()`
     to the beginning of your training script as well as your evaluation
@@ -600,13 +643,16 @@ def init(
     Call `wandb.init()` to start a run before logging data with `wandb.log()`.
 
     `wandb.init()` returns a run object, and you can also access the run object
-    with wandb.run.
+    with `wandb.run`.
 
-    At the end of your script, we will automatically call `wandb.finish(`) to
-    finalize and cleanup the run. However, if you call `wandb.init()` from a
-    child process, you must explicitly call `wandb.finish()` at the end of the
+    At the end of your script, we will automatically call `wandb.finish` to
+    finalize and cleanup the run. However, if you call `wandb.init` from a
+    child process, you must explicitly call `wandb.finish` at the end of the
     child process.
 
+    For more on using `wandb.init()`, including code snippets, check out our
+    [guide and FAQs](https://docs.wandb.ai/guides/track/launch).
+
     Arguments:
         project: (str, optional) The name of the project where you're sending
             the new run. If the project is not specified, the run is put in an
@@ -617,30 +663,30 @@ def init(
             starting to log runs.
             If you don't specify an entity, the run will be sent to your default
             entity, which is usually your username. Change your default entity
-            in [Settings](wandb.ai/settings) under "default location to create
-            new projects".
+            in [your settings](https://wandb.ai/settings) under "default location
+            to create new projects".
         config: (dict, argparse, absl.flags, str, optional)
-            This sets wandb.config, a dictionary-like object for saving inputs
+            This sets `wandb.config`, a dictionary-like object for saving inputs
             to your job, like hyperparameters for a model or settings for a data
             preprocessing job. The config will show up in a table in the UI that
             you can use to group, filter, and sort runs. Keys should not contain
             `.` in their names, and values should be under 10 MB.
             If dict, argparse or absl.flags: will load the key value pairs into
-                the wandb.config object.
+                the `wandb.config` object.
             If str: will look for a yaml file by that name, and load config from
-                that file into the wandb.config object.
+                that file into the `wandb.config` object.
         save_code: (bool, optional) Turn this on to save the main script or
             notebook to W&B. This is valuable for improving experiment
             reproducibility and to diff code across experiments in the UI. By
             default this is off, but you can flip the default behavior to on
-            in [Settings](wandb.ai/settings).
+            in [your settings page](https://wandb.ai/settings).
         group: (str, optional) Specify a group to organize individual runs into
             a larger experiment. For example, you might be doing cross
             validation, or you might have multiple jobs that train and evaluate
             a model against different test sets. Group gives you a way to
             organize runs together into a larger whole, and you can toggle this
-            on and off in the UI. For more details, see
-            [Grouping](docs.wandb.com/library/grouping).
+            on and off in the UI. For more details, see our
+            [guide to grouping runs](https://docs.wandb.com/library/grouping).
         job_type: (str, optional) Specify the type of run, which is useful when
             you're grouping runs together into larger experiments using group.
             For example, you might have multiple jobs in a group, with job types
@@ -657,13 +703,13 @@ def init(
             table to charts. Keeping these run names short makes the chart
             legends and tables easier to read. If you're looking for a place to
             save your hyperparameters, we recommend saving those in config.
-        notes: (str, optional) A longer description of the run, like a -m commit
+        notes: (str, optional) A longer description of the run, like a `-m` commit
             message in git. This helps you remember what you were doing when you
             ran this run.
         dir: (str, optional) An absolute path to a directory where metadata will
             be stored. When you call `download()` on an artifact, this is the
             directory where downloaded files will be saved. By default this is
-            the ./wandb directory.
+            the `./wandb` directory.
         resume: (bool, str, optional) Sets the resuming behavior. Options:
             `"allow"`, `"must"`, `"never"`, `"auto"` or `None`. Defaults to `None`.
             Cases:
@@ -682,9 +728,10 @@ def init(
                 `WANDB_RUN_ID="UNIQUE_ID"` and it is identical to a previous run,
                 wandb will automatically resume the run with the id. Otherwise
                 wandb will crash.
-            See https://docs.wandb.com/library/advanced/resuming for more.
+            See [our guide to resuming runs](https://docs.wandb.com/library/advanced/resuming)
+            for more.
         reinit: (bool, optional) Allow multiple `wandb.init()` calls in the same
-            process. (default: False)
+            process. (default: `False`)
         magic: (bool, dict, or str, optional) The bool controls whether we try to
             auto-instrument your script, capturing basic details of your run
             without you having to add more wandb code. (default: `False`)
@@ -713,16 +760,16 @@ def init(
             logged in to W&B. If `False`, this will let the script run in offline
             mode if a user isn't logged in to W&B. (default: `False`)
         sync_tensorboard: (bool, optional) Synchronize wandb logs from tensorboard or
-            tensorboardX and saves the relevant events file. (default: `False`)
-        monitor_gym: (bool, optional) Automatically logs videos of environment when
+            tensorboardX and save the relevant events file. (default: `False`)
+        monitor_gym: (bool, optional) Automatically log videos of environment when
             using OpenAI Gym. (default: `False`)
-            See https://docs.wandb.com/library/integrations/openai-gym
+            See [our guide to this integration](https://docs.wandb.com/library/integrations/openai-gym).
         id: (str, optional) A unique ID for this run, used for resuming. It must
             be unique in the project, and if you delete a run you can't reuse
             the ID. Use the name field for a short descriptive name, or config
             for saving hyperparameters to compare across runs. The ID cannot
             contain special characters.
-            See https://docs.wandb.com/library/resuming
+            See [our guide to resuming runs](https://docs.wandb.com/library/resuming).
 
 
     Examples:
@@ -746,6 +793,10 @@ def init(
         A `Run` object.
     """
     wandb._assert_is_user_process()
+
+    if resume is True:
+        resume = "auto"  # account for changing resume interface, True and auto should behave the same
+
     kwargs = dict(locals())
     error_seen = None
     except_exit = None
@@ -770,7 +821,8 @@ def init(
                 pass
             # TODO(jhr): figure out how to make this RunDummy
             run = None
-    except UsageError:
+    except UsageError as e:
+        wandb.termerror(str(e))
         raise
     except KeyboardInterrupt as e:
         assert logger
