@@ -6,7 +6,9 @@ account.
 """
 from __future__ import print_function
 
+import enum
 import os
+from typing import Dict, Optional, Tuple
 
 import click
 import wandb
@@ -19,10 +21,6 @@ from .wandb_settings import Settings
 from ..apis import InternalApi
 
 
-if wandb.TYPE_CHECKING:  # type: ignore
-    from typing import Dict, Optional  # noqa: F401 pylint: disable=unused-import
-
-
 def _handle_host_wandb_setting(host: Optional[str], cloud: bool = False) -> None:
     """Write the host parameter from wandb.login or wandb login to
     the global settings file so that it is used automatically by
@@ -39,7 +37,7 @@ def _handle_host_wandb_setting(host: Optional[str], cloud: bool = False) -> None
         _api.set_setting("base_url", host, globally=True, persist=True)
 
 
-def login(anonymous=None, key=None, relogin=None, host=None, force=None):
+def login(anonymous=None, key=None, relogin=None, host=None, force=None, timeout=None):
     """
     Log in to W&B.
 
@@ -51,6 +49,7 @@ def login(anonymous=None, key=None, relogin=None, host=None, force=None):
         key: (string, optional) authentication key.
         relogin: (bool, optional) If true, will re-prompt for API key.
         host: (string, optional) The host to connect to.
+        timeout: (int, optional) Number of seconds to wait for user input.
 
     Returns:
         bool: if key is configured
@@ -67,6 +66,13 @@ def login(anonymous=None, key=None, relogin=None, host=None, force=None):
     return True if configured else False
 
 
+class ApiKeyStatus(enum.Enum):
+    VALID = 1
+    NOTTY = 2
+    OFFLINE = 3
+    DISABLED = 4
+
+
 class _WandbLogin(object):
     def __init__(self):
         self.kwargs: Optional[Dict] = None
@@ -75,6 +81,7 @@ class _WandbLogin(object):
         self._silent = None
         self._wl = None
         self._key = None
+        self._relogin = None
 
     def setup(self, kwargs):
         self.kwargs = kwargs
@@ -85,6 +92,8 @@ class _WandbLogin(object):
         if settings_param:
             login_settings._apply_settings(settings_param)
         _logger = wandb.setup()._get_logger()
+        # Do not save relogin into settings as we just want to relogin once
+        self._relogin = kwargs.pop("relogin", None)
         login_settings._apply_login(kwargs, _logger=_logger)
 
         # make sure they are applied globally
@@ -102,7 +111,7 @@ class _WandbLogin(object):
 
     def login(self):
         apikey_configured = self.is_apikey_configured()
-        if self._settings.relogin:
+        if self._settings.relogin or self._relogin:
             apikey_configured = False
         if not apikey_configured:
             return False
@@ -146,10 +155,18 @@ class _WandbLogin(object):
         self.update_session(key)
         self._key = key
 
-    def update_session(self, key):
+    def update_session(
+        self, key: Optional[str], status: ApiKeyStatus = ApiKeyStatus.VALID
+    ) -> None:
         _logger = wandb.setup()._get_logger()
         settings: Settings = wandb.Settings()
-        login_settings = dict(api_key=key) if key else dict(mode="offline")
+        login_settings = dict()
+        if status == ApiKeyStatus.OFFLINE:
+            login_settings = dict(mode="offline")
+        elif status == ApiKeyStatus.DISABLED:
+            login_settings = dict(mode="disabled")
+        elif key:
+            login_settings = dict(api_key=key)
         settings._apply_source_login(login_settings, _logger=_logger)
         self._wl._update(settings=settings)
         # Whenever the key changes, make sure to pull in user settings
@@ -157,15 +174,33 @@ class _WandbLogin(object):
         if not self._wl.settings._offline:
             self._wl._update_user_settings()
 
-    def prompt_api_key(self):
+    def _prompt_api_key(self) -> Tuple[Optional[str], ApiKeyStatus]:
+
         api = Api(self._settings)
-        key = apikey.prompt_api_key(
-            self._settings,
-            api=api,
-            no_offline=self._settings.force,
-            no_create=self._settings.force,
-        )
+        try:
+            key = apikey.prompt_api_key(
+                self._settings,
+                api=api,
+                no_offline=self._settings.force if self._settings else None,
+                no_create=self._settings.force if self._settings else None,
+            )
+        except ValueError as e:
+            # invalid key provided, try again
+            wandb.termerror(e.args[0])
+            return self._prompt_api_key()
+
+        except TimeoutError:
+            wandb.termlog("W&B disabled due to login timeout.")
+            return None, ApiKeyStatus.DISABLED
         if key is False:
+            return None, ApiKeyStatus.NOTTY
+        if not key:
+            return None, ApiKeyStatus.OFFLINE
+        return key, ApiKeyStatus.VALID
+
+    def prompt_api_key(self):
+        key, status = self._prompt_api_key()
+        if status == ApiKeyStatus.NOTTY:
             directive = (
                 "wandb login [your_api_key]"
                 if self._settings._cli_only_mode
@@ -173,7 +208,7 @@ class _WandbLogin(object):
             )
             raise UsageError("api_key not configured (no-tty). call " + directive)
 
-        self.update_session(key)
+        self.update_session(key, status=status)
         self._key = key
 
     def propogate_login(self):
@@ -191,6 +226,7 @@ def _login(
     relogin=None,
     host=None,
     force=None,
+    timeout=None,
     _backend=None,
     _silent=None,
     _disable_warning=None,
