@@ -5,14 +5,11 @@ metric.
 """
 
 import logging
+from typing import Callable, Optional, Sequence, Tuple
 
-import wandb
 from wandb.proto import wandb_internal_pb2 as pb
 
 
-if wandb.TYPE_CHECKING:
-    from typing import Callable, Optional, Sequence, Tuple
-
 logger = logging.getLogger("wandb")
 
 
