@@ -61,12 +61,14 @@ def plot_classifier(
     is_binary=False,
     model_name="Classifier",
     feature_names=None,
+    log_learning_curve=False,
 ):
     """
     Generates all sklearn classifier plots supported by W&B.
         The following plots are generated:
-        feature importances, learning curve, confusion matrix, summary metrics,
-        class balance plot, calibration curve, roc curve & precision recall curve.
+        feature importances, confusion matrix, summary metrics,
+        class balance plot, calibration curve, roc curve, precision-recall curve,
+        and optionally learning curve.
 
     Should only be called with a fitted classifer (otherwise an error is thrown).
 
@@ -86,6 +88,8 @@ def plot_classifier(
         model_name (str): Model name. Defaults to 'Classifier'
         feature_names (list): Names for features. Makes plots easier to read by
                                 replacing feature indexes with corresponding names.
+        log_learning_curve (bool): Whether or not to log the learning curve.
+                                    Defaults to False.
 
     Returns:
         Nothing. To see plots, go to your W&B run page then expand the 'media' tab
@@ -97,10 +101,12 @@ def plot_classifier(
                         'RandomForest', ['barks', 'drools, 'plays_fetch', 'breed'])
     """
     wandb.termlog("\nPlotting %s." % model_name)
-    plot_feature_importances(model, feature_names)
-    wandb.termlog("Logged feature importances.")
-    plot_learning_curve(model, X_train, y_train)
-    wandb.termlog("Logged learning curve.")
+    if not isinstance(model, naive_bayes.MultinomialNB):
+        plot_feature_importances(model, feature_names)
+        wandb.termlog("Logged feature importances.")
+    if log_learning_curve:
+        plot_learning_curve(model, X_train, y_train)
+        wandb.termlog("Logged learning curve.")
     plot_confusion_matrix(y_test, y_pred, labels)
     wandb.termlog("Logged confusion matrix.")
     plot_summary_metrics(model, X=X_train, y=y_train, X_test=X_test, y_test=y_test)
@@ -109,7 +115,7 @@ def plot_classifier(
     wandb.termlog("Logged class proportions.")
     if not isinstance(model, naive_bayes.MultinomialNB):
         plot_calibration_curve(model, X_train, y_train, model_name)
-    wandb.termlog("Logged calibration curve.")
+        wandb.termlog("Logged calibration curve.")
     plot_roc(y_test, y_probas, labels)
     wandb.termlog("Logged roc curve.")
     plot_precision_recall(y_test, y_probas, labels)
@@ -398,23 +404,23 @@ def plot_roc(
     classes_to_plot=None,
 ):
     """
-     Logs the plots generated by roc() to wandb.
+    Logs the plots generated by roc() to wandb.
 
-     Arguments:
-         y_true (arr): Test set labels.
-         y_probas (arr): Test set predicted probabilities.
-         labels (list): Named labels for target varible (y). Makes plots easier to
-                         read by replacing target values with corresponding index.
-                         For example labels= ['dog', 'cat', 'owl'] all 0s are
-                         replaced by 'dog', 1s by 'cat'.
+    Arguments:
+        y_true (arr): Test set labels.
+        y_probas (arr): Test set predicted probabilities.
+        labels (list): Named labels for target varible (y). Makes plots easier to
+                        read by replacing target values with corresponding index.
+                        For example labels= ['dog', 'cat', 'owl'] all 0s are
+                        replaced by 'dog', 1s by 'cat'.
 
-     Returns:
-         Nothing. To see plots, go to your W&B run page then expand the 'media' tab
-               under 'auto visualizations'.
+    Returns:
+        Nothing. To see plots, go to your W&B run page then expand the 'media' tab
+              under 'auto visualizations'.
 
-     Example:
-         wandb.sklearn.plot_roc(y_true, y_probas, labels)
-     """
+    Example:
+        wandb.sklearn.plot_roc(y_true, y_probas, labels)
+    """
     wandb.log(
         {"roc": roc(y_true, y_probas, labels, plot_micro, plot_macro, classes_to_plot)}
     )
@@ -600,7 +606,7 @@ def plot_feature_importances(
     Example:
         wandb.sklearn.plot_feature_importances(model, ['width', 'height, 'length'])
     """
-    attributes_to_check = ["feature_importances_", "coef_"]
+    attributes_to_check = ["feature_importances_", "feature_log_prob_", "coef_"]
 
     def get_attributes_as_formatted_string():
         result = ""
@@ -629,10 +635,27 @@ def plot_feature_importances(
     if test_missing(model=model) and test_types(model=model) and test_fitted(model):
         if found_attribute == "feature_importances_":
             importances = model.feature_importances_
-        if found_attribute == "coef_":  # ElasticNet or ElasticNetCV like models
+        elif found_attribute == "coef_":  # ElasticNet or ElasticNetCV like models
             importances = model.coef_
+        elif found_attribute == "feature_log_prob_":
+            # coef_ was deprecated in sklearn 0.24, replaced with
+            # feature_log_prob_
+            importances = model.feature_log_prob_
+
+        if len(importances.shape) > 1:
+            if np.prod(importances.shape) > importances.shape[0]:
+                nd = len(importances.shape)
+                wandb.termwarn(
+                    f"{nd}-dimensional feature importances array passed to plot_feature_importances. "
+                    f"{nd}-dimensional and higher feature importances arrays are not currently supported. "
+                    f"These importances will not be plotted."
+                )
+                return
+            else:
+                importances = np.squeeze(importances)
 
         indices = np.argsort(importances)[::-1]
+        importances = importances[indices]
 
         if not feature_names:
             feature_names = indices
