@@ -8,9 +8,10 @@ import colorsys
 import contextlib
 import codecs
 import errno
+import functools
+import gzip
 import hashlib
 import json
-import getpass
 import logging
 import math
 import numbers
@@ -19,46 +20,39 @@ import os
 import re
 import shlex
 import socket
-import subprocess
 import sys
 import threading
 import time
 import random
-import stat
 import shortuuid
 import importlib
+import tarfile
+import tempfile
 import types
 import yaml
 from datetime import date, datetime
 import platform
-from six.moves.urllib.parse import urlparse
+from six.moves import urllib
 
-import click
 import requests
 import six
 from six.moves import queue, input
-import textwrap
 from sys import getsizeof
-from collections import namedtuple
 from six.moves.collections_abc import Mapping, Sequence
 from importlib import import_module
 import sentry_sdk
 from sentry_sdk import capture_exception
 from sentry_sdk import capture_message
-from sentry_sdk import configure_scope
-from wandb.env import error_reporting_enabled
+from wandb.env import error_reporting_enabled, get_app_url
 
 import wandb
-from wandb.errors import CommError, term
-from wandb.old.core import wandb_dir
 from wandb import env
-
-from typing import List
+from wandb.errors import CommError, term
 
 logger = logging.getLogger(__name__)
 _not_importable = set()
 
-MAX_LINE_SIZE = 9 * 1024 * 1024 - 100 * 1024  # imposed by back end
+MAX_LINE_BYTES = (10 << 20) - (100 << 10)  # imposed by back end
 IS_GIT = os.path.exists(os.path.join(os.path.dirname(__file__), "..", ".git"))
 
 # these match the environments for gorilla
@@ -273,15 +267,20 @@ VALUE_BYTES_LIMIT = 100000
 
 
 def app_url(api_url):
+    """Returns the frontend app url without a trailing slash."""
+    # TODO: move me to settings
+    app_url = get_app_url()
+    if app_url is not None:
+        return app_url.strip("/")
     if "://api.wandb.test" in api_url:
         # dev mode
-        return api_url.replace("://api.", "://app.")
+        return api_url.replace("://api.", "://app.").strip("/")
     elif "://api.wandb." in api_url:
         # cloud
-        return api_url.replace("://api.", "://")
+        return api_url.replace("://api.", "://").strip("/")
     elif "://api." in api_url:
         # onprem cloud
-        return api_url.replace("://api.", "://app.")
+        return api_url.replace("://api.", "://app.").strip("/")
     # wandb/local
     return api_url
 
@@ -307,6 +306,83 @@ def get_h5_typename(o):
         return o.__class__.__module__.split(".")[0] + "." + o.__class__.__name__
 
 
+def is_uri(string):
+    parsed_uri = urllib.parse.urlparse(string)
+    return len(parsed_uri.scheme) > 0
+
+
+def local_file_uri_to_path(uri):
+    """
+    Convert URI to local filesystem path.
+    No-op if the uri does not have the expected scheme.
+    """
+    path = urllib.parse.urlparse(uri).path if uri.startswith("file:") else uri
+    return urllib.request.url2pathname(path)
+
+
+def get_local_path_or_none(path_or_uri):
+    """Check if the argument is a local path (no scheme or file:///) and return local path if true,
+    None otherwise.
+    """
+    parsed_uri = urllib.parse.urlparse(path_or_uri)
+    if (
+        len(parsed_uri.scheme) == 0
+        or parsed_uri.scheme == "file"
+        and len(parsed_uri.netloc) == 0
+    ):
+        return local_file_uri_to_path(path_or_uri)
+    else:
+        return None
+
+
+def make_tarfile(output_filename, source_dir, archive_name, custom_filter=None):
+    # Helper for filtering out modification timestamps
+    def _filter_timestamps(tar_info):
+        tar_info.mtime = 0
+        return tar_info if custom_filter is None else custom_filter(tar_info)
+
+    unzipped_filename = tempfile.mktemp()
+    try:
+        with tarfile.open(unzipped_filename, "w") as tar:
+            tar.add(source_dir, arcname=archive_name, filter=_filter_timestamps)
+        # When gzipping the tar, don't include the tar's filename or modification time in the
+        # zipped archive (see https://docs.python.org/3/library/gzip.html#gzip.GzipFile)
+        with gzip.GzipFile(
+            filename="", fileobj=open(output_filename, "wb"), mode="wb", mtime=0
+        ) as gzipped_tar, open(unzipped_filename, "rb") as tar:
+            gzipped_tar.write(tar.read())
+    finally:
+        os.remove(unzipped_filename)
+
+
+def _user_args_to_dict(arguments):
+    user_dict = {}
+    i = 0
+    while i < len(arguments):
+        arg = arguments[i]
+        split = arg.split("=", maxsplit=1)
+        # flag arguments don't require a value -> set to True if specified
+        if len(split) == 1 and (
+            i + 1 >= len(arguments) or arguments[i + 1].startswith("-")
+        ):
+            name = split[0].lstrip("-")
+            value = True
+            i += 1
+        elif len(split) == 1 and not arguments[i + 1].startswith("-"):
+            name = split[0].lstrip("-")
+            value = arguments[i + 1]
+            i += 2
+        elif len(split) == 2:
+            name = split[0].lstrip("-")
+            value = split[1]
+            i += 1
+        if name in user_dict:
+            wandb.termerror("Repeated parameter: '%s'" % name)
+            sys.exit(1)
+        user_dict[name] = value
+    return user_dict
+
+
 def is_tf_tensor(obj):
     import tensorflow
 
@@ -335,6 +411,16 @@ def is_pytorch_tensor_typename(typename):
     )
 
 
+def is_jax_tensor_typename(typename):
+    return typename.startswith("jaxlib.") and "DeviceArray" in typename
+
+
+def get_jax_tensor(obj):
+    import jax
+
+    return jax.device_get(obj)
+
+
 def is_fastai_tensor_typename(typename):
     return typename.startswith("fastai.") and ("Tensor" in typename)
 
@@ -450,6 +536,8 @@ def json_friendly(obj):
             obj = obj.cpu().detach().numpy()
         else:
             return obj.item(), True
+    elif is_jax_tensor_typename(typename):
+        obj = get_jax_tensor(obj)
 
     if is_numpy_array(obj):
         if obj.size == 1:
@@ -580,10 +668,10 @@ def launch_browser(attempt_launch_browser=True):
     return launch_browser
 
 
-def generate_id():
+def generate_id(length=8):
     # ~3t run ids (36**8)
     run_gen = shortuuid.ShortUUID(alphabet=list("0123456789abcdefghijklmnopqrstuvwxyz"))
-    return run_gen.random(8)
+    return run_gen.random(length)
 
 
 def parse_tfjob_config():
@@ -713,7 +801,7 @@ def no_retry_auth(e):
     if e.response is None:
         return True
     # Don't retry bad request errors; raise immediately
-    if e.response.status_code == 400:
+    if e.response.status_code in (400, 409):
         return False
     # Retry all non-forbidden/unauthorized/not-found errors.
     if e.response.status_code not in (401, 403, 404):
@@ -964,21 +1052,36 @@ def class_colors(class_count):
     ]
 
 
-def _prompt_choice():
-    try:
-        return int(input("%s: Enter your choice: " % term.LOG_STRING)) - 1  # noqa: W503
-    except ValueError:
-        return -1
+def _prompt_choice(input_timeout: int = None) -> str:
+    input_fn = input
+    prompt = term.LOG_STRING
+    if input_timeout:
+        # delayed import to mitigate risk of timed_input complexity
+        from wandb.sdk.lib import timed_input
+
+        input_fn = functools.partial(timed_input.timed_input, timeout=input_timeout)
+        # timed_input doesnt handle enhanced prompts
+        if platform.system() == "Windows":
+            prompt = "wandb"
+    choice = input_fn(f"{prompt}: Enter your choice: ")
+    return choice
 
 
-def prompt_choices(choices, allow_manual=False):
+def prompt_choices(choices, allow_manual=False, input_timeout: int = None):
     """Allow a user to choose from a list of options"""
     for i, choice in enumerate(choices):
         wandb.termlog("(%i) %s" % (i + 1, choice))
 
     idx = -1
     while idx < 0 or idx > len(choices) - 1:
-        idx = _prompt_choice()
+        choice = _prompt_choice(input_timeout=input_timeout)
+        if not choice:
+            continue
+        idx = -1
+        try:
+            idx = int(choice) - 1
+        except ValueError:
+            pass
         if idx < 0 or idx > len(choices) - 1:
             wandb.termwarn("Invalid choice")
     result = choices[idx]
@@ -1188,16 +1291,23 @@ def hex_to_b64_id(encoded_string):
 
 def host_from_path(path):
     """returns the host of the path"""
-    url = urlparse(path)
+    url = urllib.parse.urlparse(path)
     return url.netloc
 
 
 def uri_from_path(path):
     """returns the URI of the path"""
-    url = urlparse(path)
+    url = urllib.parse.urlparse(path)
     return url.path if url.path[0] != "/" else url.path[1:]
 
 
+def is_unicode_safe(stream):
+    """returns true if the stream supports UTF-8"""
+    if not hasattr(stream, "encoding"):
+        return False
+    return stream.encoding == "UTF-8"
+
+
 def _has_internet():
     """Attempts to open a DNS connection to Googles root servers"""
     try:
@@ -1259,8 +1369,36 @@ def _is_databricks():
     return False
 
 
+def sweep_config_err_text_from_jsonschema_violations(violations):
+    """Consolidate violation strings from wandb/sweeps describing the ways in which a
+    sweep config violates the allowed schema as a single string.
+
+    Parameters
+    ----------
+    violations: list of str
+        The warnings to render.
+
+    Returns
+    -------
+    violation: str
+        The consolidated violation text.
+
+    """
+
+    violation_base = (
+        "Malformed sweep config detected! This may cause your sweep to behave in unexpected ways.\n"
+        "To avoid this, please fix the sweep config schema violations below:"
+    )
+
+    for i, warning in enumerate(violations):
+        violations[i] = "  Violation {}. {}".format(i + 1, warning)
+    violation = "\n".join([violation_base] + violations)
+
+    return violation
+
+
 def handle_sweep_config_violations(warnings):
-    """Render warnings from gorilla describing the ways in which a 
+    """Render warnings from gorilla describing the ways in which a
     sweep config violates the allowed schema as terminal warnings.
 
     Parameters
@@ -1269,15 +1407,7 @@ def handle_sweep_config_violations(warnings):
         The warnings to render.
     """
 
-    warning_base = (
-        "Malformed sweep config detected! This may cause your sweep to behave in unexpected ways.\n"
-        "To avoid this, please fix the sweep config schema violations below:"
-    )
-
-    for i, warning in enumerate(warnings):
-        warnings[i] = "  Violation {}. {}".format(i + 1, warning)
-    warning = "\n".join([warning_base] + warnings)
-
+    warning = sweep_config_err_text_from_jsonschema_violations(warnings)
     if len(warnings) > 0:
         term.termwarn(warning)
 
